package codec

import (
	"bytes"
	"encoding/binary"

	"github.com/sirupsen/logrus"
)

type Cmd struct {
	Query     string
	Msg       Fields
	PayloadID uint32
}

func NewCmd(pkt *RawPacket) (*Cmd, error) {
	l := len(pkt.Payload)
	if l > 8096 {
		l = 8096
	}

	logrus.
		WithField("query", string(pkt.Query)).
		WithField("payload", string(pkt.Payload[:l])).
		WithField("broadcast", pkt.Broadcast[0]).
		WithField("id", pkt.Broadcast[1:]).
		Debug("codec.NewCmd")

	out := &Cmd{
		Query:     string(pkt.Query),
		PayloadID: binary.BigEndian.Uint32(pkt.Broadcast),
		Msg:       DecodeFESL(pkt.Payload),
	}

	return out, nil
}

func ParseCommands(data []byte) ([]*Cmd, error) {
	buf := bytes.NewBuffer(data)
	cmds := []*Cmd{}

	for buf.Len() > 0 {
		pkt, err := ExtractPacket(buf)
		if err != nil {
			return nil, err
		}

		cmd, err := NewCmd(pkt)
		if err != nil {
			return nil, err
		}

		cmds = append(cmds, cmd)
	}

	return cmds, nil
}
