package acct

import (
	uuid "github.com/satori/go.uuid"
	"github.com/sirupsen/logrus"

	"gitlab.com/oiacow/goheroes/backend/core"
)

type reqNuLogin struct {
	TXN                 string `fesl:"TXN"`
	ReturnEncryptedInfo int    `fesl:"returnEncryptedInfo"`
	MacAddr             string `fesl:"macAddr"`
}

type reqNuLoginServer struct {
	reqNuLogin

	AccountName     string `fesl:"nuid"`     // Value specified in +eaAccountName
	AccountPassword string `fesl:"password"` // Value specified in +eaAccountPassword
}

type reqNuLoginClient struct {
	reqNuLogin

	EncryptedInfo string `fesl:"encryptedInfo"` // Value specified in +sessionId
}

type ansNuLogin struct {
	Txn       string `fesl:"TXN"`
	ProfileID int    `fesl:"profileId"`
	UserID    int    `fesl:"userId"`
	NucleusID int    `fesl:"nuid"`
	LobbyKey  string `fesl:"lkey"`
}

type ansNuLoginErr struct {
	Txn    string                `fesl:"TXN"`
	Msg    string                `fesl:"localizedMessage"`
	Errors []nuLoginContainerErr `fesl:"errorContainer"`
	Code   int                   `fesl:"errorCode"`
}

type nuLoginContainerErr struct {
	Value      string `fesl:"value"`
	FieldError string `fesl:"fieldError"`
	FieldName  string `fesl:"fieldName"`
}

//if you think this is difficult go learn c++
func randStr() (rand string, retErr error) {
	newRandom := uuid.NewV4().String()
	rand = newRandom
	return rand, nil
}

// NuLogin handles acct.NuLogin cmd
func (acct *Account) NuLogin(event core.EClientCmd) {
	lkey, _ := randStr()
	event.Client.PlayerData.LobbyKey = lkey
	err := core.Lobby.Add(lkey, event.Client.PlayerData)
	if err != nil {
		logrus.WithError(err).Warn("Cant add ClientData in acct.NuLogin")
		return
	}

	switch event.Client.GetClientType() {
	case clientTypeServer:
		acct.serverNuLogin(event)
	default:
		acct.clientNuLogin(event)
	}
}

func (acct *Account) clientNuLogin(event core.EClientCmd) {
	player, err := acct.DB.GetPlayerByToken(
		acct.DB.NewSession(),
		event.Cmd.Msg["encryptedInfo"],
	)
	if err != nil {
		logrus.WithError(err).Warn("Client cant sign in the acct.NuLogin")
		acct.clientLoginErr(&event)
		return
	}

	event.Client.PlayerData.PlayerID = player.ID
	acct.answer(
		event.Client,
		event.Cmd.PayloadID,
		ansNuLogin{
			Txn:       acctNuLogin,
			UserID:    player.ID,
			ProfileID: player.ID,
			NucleusID: player.ID,
			LobbyKey:  event.Client.PlayerData.LobbyKey,
		},
	)
}

func (acct *Account) clientLoginErr(event *core.EClientCmd) {
	acct.answer(
		event.Client,
		event.Cmd.PayloadID,
		ansNuLoginErr{
			Txn:  acctNuLogin,
			Msg:  `" "`, //empty against spoofers
			Code: 120,
		},
	)
}

// acctNuLoginServer - login cmd for servers
func (acct *Account) serverNuLogin(event core.EClientCmd) {
	gameServer, err := acct.DB.GetServerLogin(
		acct.DB.NewSession(),
		event.Cmd.Msg["nuid"],
	)
	if err != nil {
		logrus.WithError(err).Warn("Server cant sign in the acct.NuLogin")
		acct.serverLoginErr(&event)
		return
	}

	//todo create sessionID here -> send to mysql
	//instead of receiving a raw one from mysql
	if gameServer.AccountPassword != event.Cmd.Msg["password"] {
		acct.serverLoginErr(&event)
		return
	}
	DataPkt := event.Client.PlayerData
	DataPkt.ServerID = gameServer.ID
	DataPkt.ServerSoldierName = gameServer.SoldierName
	DataPkt.ServerUserName = gameServer.AccountUsername

	acct.answer(
		event.Client,
		event.Cmd.PayloadID,
		ansNuLogin{
			Txn:       acctNuLogin,
			ProfileID: gameServer.ID,
			UserID:    gameServer.ID,
			NucleusID: gameServer.ID,
			LobbyKey:  event.Client.PlayerData.LobbyKey,
		},
	)
}

func (acct *Account) serverLoginErr(event *core.EClientCmd) {
	acct.answer(
		event.Client,
		event.Cmd.PayloadID,
		ansNuLoginErr{
			Txn:  acctNuLogin,
			Msg:  `" "`, //empty against spoofers
			Code: 122,
		},
	)
}
