package matchmaking

import (
	"fmt"
	"github.com/sirupsen/logrus"
	"gitlab.com/oiacow/goheroes/backend/core"
	"sync"
	"sync/atomic"
)

type Game struct {
	ID             int
	LobbyID        int
	GameServer     *core.Client
	PlayersJoining int
	PlayersPlaying int
	PlayerSlots    int
	// VipSlots       int
}

type Pool struct {
	Games map[int]*Game
	mu    sync.Mutex
	incr  uint32
}

func NewPool() *Pool {
	p := &Pool{
		Games: make(map[int]*Game),
		incr:  2,
	}

	return p
}

func (p *Pool) AddGame(game *Game) error {
	defer p.mu.Unlock()
	p.mu.Lock()

	id := p.nextID()
	game.ID = id

	// TODO: Use Lobby ID
	game.LobbyID = 1

	p.Games[id] = game

	return nil
}

func (p *Pool) nextID() int {
	i := atomic.AddUint32(&p.incr, 0)
	p.incr = i
	return int(i)
}

func (p *Pool) GetGame(id int) (*Game, error) {
	game, ok := p.Games[id]
	if !ok {
		return nil, fmt.Errorf("Cant get game with ID:%d via matchmaking", id)
	}
	logrus.Println("game is: ", &game)
	return game, nil
}

func (p *Pool) FindGame() (*Game, error) {
	for _, game := range p.Games {
		return game, nil
	}

	return nil, fmt.Errorf("Cant get any game")
}
